import type { Context, GeneratorOptions, InternalOptions } from "./types/index";
/**
 * Base abstract class for for pg-generator classes.
 */
export declare abstract class PgGenerator<O extends GeneratorOptions = GeneratorOptions> {
    #private;
    protected options: O & {
        envPrefix: string;
        context: Record<string, any>;
    };
    /**
     * Creates an instance of PgGenerator.
     *
     * @param options are parameters for several options.
     * @param internalOptions are added by `pg-generator` functions and not for user.
     */
    constructor(options: O, internalOptions: InternalOptions);
    /**
     * Renders all templates in `[rootDir]/templates` directory with the render function using related context data,
     * and writes generated contents to the files in output directory. If render function returns `undefined`
     * for a template/database object pair no file is not written for that pair.
     *
     * Template file names may contain variables and basic filter functions to change names of generated files.
     *
     * Additionally copies all files in `[rootDir]/files` to the output directory.
     */
    generate(): Promise<void>;
    /**
     * If clear option is true, this method is called. It deletes destination path.
     * User may override this method to tailor clear operation according to generator's
     * purposes.
     */
    protected clear(): Promise<any>;
    /**
     * Executed after clear operation.
     */
    protected init(): Promise<any> | any;
    /**
     * Provides generator specific extra context to templates. This data is deeply merged with
     * context provided by user.
     *
     * @returns generator spesific extra context data.
     *
     * @example
     * export default class App extends PgGenerator {
     *   protected context(): Record<string, any> {
     *     return { global: { addSchemaName: true } };
     *   }
     * }
     */
    protected context(): Promise<Record<string, any>> | Record<string, any>;
    /**
     * Generates content from the template with provided path using the context.
     *
     * @param templatePath is the path of the template file to generate content.
     * @param context is the data to be used for generating content.
     * @returns generated content.
     *
     * @example
     * import engine from "my-favorite-template-engine";
     *
     * export default class Md extends PgGenerator {
     *   protected async render(templatePath: string, context: Context): Promise<string> {
     *     return engine.render(templatePath, context);
     *   }
     * }
     */
    protected render(templatePath: string, context: Context): Promise<undefined | string> | undefined | string;
    /**
     * Formats given content. By default `prettier` is used. Plase note that at this stage file is not written to disk yet.
     *
     * @param destination is the path of the destination file to be created.
     * @param content is the content to format.
     * @returns formatted code.
     */
    protected format(destination: string, content?: string): Promise<undefined | string> | undefined | string;
    /**
     * Writes generated content from [[render]] to the destination file.
     *
     * @param destination is the destionation path of the file to be cerated..
     * @param content is the content to process.
     */
    protected process(destination: string, content: string): Promise<any> | any;
    /**
     * Logs given messsage if `options.log` is true. Also outputs colorized paths for
     * source and destination.
     *
     * @param title is the title of the log.
     * @param message is the nessage to log.
     * @param options are options.
     * @param options.message is the message to log.
     * @param options.source is the source path to log.
     * @param options.destination is the destination path to log.
     *
     * @hidden
     */
    protected log(title: string, { message, source, destination }: {
        message?: string;
        source?: string;
        destination?: string;
    }): void;
    /**
     * Returns abosulte path for the given path relative to output directory.
     *
     * @param path is the path relative to output directory.
     * @returns the absolute path.
     */
    protected destinationPath(path?: string): string;
    /**
     * Returns abosulte path for the given path relative to template root.
     *
     * @param path is the path relative to template root.
     * @returns the absolute path.
     */
    protected templatePath(path?: string): string;
    /**
     * Executes the default sub-generator `app` from a generator. If options are provided, they are combined with current options.
     *
     * @param generator is the name or path of the generator. If it is a local path, use `require.resolve` or provide an absolute path.
     * @param options are the new options added on top of curent options.
     *
     * @throws error if generator can not be found.
     *
     * @example
     * this.composeWith("generator-from-npm");
     * this.composeWith("generator-from-npm", options);
     * this.composeWith(require.resolve("./local-generator"), options);
     */
    protected composeWith(generator: string, options?: GeneratorOptions): Promise<void>;
    /**
     * Executes a sub-generator from a generator. If options are provided, they are merged with current options. Also some of the options
     * are changed to sensible default values for sub-generators (e.g. `clear` is changed to `false`), but they can be overridden by
     * newly provided options.
     *
     * @param generator is the name or path of the generator. If it is a local path, use `require.resolve` or provide an absolute path.
     * @param subGeneratorName is the name of the sub-generator to be executed.
     * @param options are the new options added on top of curent options.
     *
     * @throws error if generator or sub-generator can not be found.
     *
     * @example
     * this.composeWith("generator-from-npm", "sub-generator");
     * this.composeWith(require.resolve("./local-generator"), "sub-generator", options);
     */
    protected composeWith(generator: string, subGeneratorName?: string, options?: GeneratorOptions): Promise<void>;
    /**
     * Writes given content to the file after creating it's directory.
     *
     * @param destinantion is the path to write file.
     * @param content is the content to write.
     */
    private writeTemplate;
    /**
     * Copy the source file to the destination. Also creates the destination directory if it does not exist.
     *
     * @param source is the source file to copy.
     * @param destination is the destination file to be created.
     */
    private copyTemplate;
    /**
     * Removes file or directory (and it's contents() from destination. If destination is equal to `cwd()`
     * it throws error and aborts operation for a safety measure. Most probably `cwd()` contains other files too.
     *
     * @param path is the path relative to the destination root.
     */
    private deleteDestination;
    /**
     * Fetches and merges context data from `pg-generator` instance, context file and options.
     * The context file may be a JSON5 data file (may contain comments etc.) or a JavaScript file
     * with a default export of object or a function. If it is a function, it is passed
     * `pg-structure` db instance.
     *
     * Merge priorities are: `options.context` > context file > `context()` method.
     *
     * @param contextFile is the file to read from.
     * @param context is the context data.
     * @returns merged context data.
     */
    private fetchContext;
}
//# sourceMappingURL=pg-generator.d.ts.map