"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _destinationRoot, _internalOptions;
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgGenerator = void 0;
/* eslint-disable class-methods-use-this */
const path_1 = require("path");
const fs_1 = require("fs");
const meow_helper_1 = require("meow-helper");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const json5_1 = __importDefault(require("json5"));
const prettier_1 = __importDefault(require("prettier"));
const pgen_error_1 = require("./utils/pgen-error");
const utils_1 = require("./utils");
const COLORS = {
    default: "green",
    delete: "yellow",
};
/**
 * Base abstract class for for pg-generator classes.
 */
class PgGenerator {
    /**
     * Creates an instance of PgGenerator.
     *
     * @param options are parameters for several options.
     * @param internalOptions are added by `pg-generator` functions and not for user.
     */
    constructor(options, internalOptions) {
        _destinationRoot.set(this, void 0);
        _internalOptions.set(this, void 0);
        if (options.clear && !options.outDir)
            throw new pgen_error_1.PgenError("As a precaution, 'clear' option cannot be used without 'outDir' option.");
        const collisions = internalOptions.db.relationNameCollisions;
        if (collisions)
            throw pgen_error_1.PgenError.collisionError(collisions);
        this.options = { envPrefix: "DB", copy: true, context: {}, ...options };
        __classPrivateFieldSet(this, _internalOptions, internalOptions);
        __classPrivateFieldSet(this, _destinationRoot, path_1.resolve(internalOptions.cwd, this.options.outDir || ""));
    }
    /**
     * Renders all templates in `[rootDir]/templates` directory with the render function using related context data,
     * and writes generated contents to the files in output directory. If render function returns `undefined`
     * for a template/database object pair no file is not written for that pair.
     *
     * Template file names may contain variables and basic filter functions to change names of generated files.
     *
     * Additionally copies all files in `[rootDir]/files` to the output directory.
     */
    async generate() {
        if (this.options.clear)
            await this.clear();
        await this.init();
        const { templates, files } = await utils_1.scanTemplateDir(__classPrivateFieldGet(this, _internalOptions).templateDir);
        const context = await this.fetchContext();
        // Generate templates. Provide absolute outdir. Don't copy files, because you should use Yeoman's `mem-fs-editor` API.
        const processTemplates = Promise.all(templates.flatMap((path) => {
            const { className, accessor, targetPath } = utils_1.parseTemplatePath(path);
            return utils_1.getDbObjects(__classPrivateFieldGet(this, _internalOptions).db, className, accessor, path).map(async (dbObject) => {
                const destinationPath = utils_1.resolvePath(targetPath, dbObject); // e.g. {name#dashcase}.js -> member-option.js
                const content = await this.format(destinationPath, await this.render(path, utils_1.augmentContext(dbObject, context)));
                if (content !== undefined && content.replace(/\s/g, "") !== "")
                    await this.process(destinationPath, content);
            });
        }));
        const copyFiles = Promise.all(files.map((file) => this.copyTemplate(file, file)));
        await Promise.all([processTemplates, copyFiles]);
    }
    //
    // ────────────────────────────────────────────────────────── I ──────────
    //   :::::: L I F E C Y C L E : :  :   :    :     :        :          :
    // ────────────────────────────────────────────────────────────────────
    //
    // Methods below are lifecycle methods and most probably would be extended
    // by child classes for custom behavior. Providing `render` is mandatory.
    // Flow: clear --> init -> context -> render -> format -> process
    /**
     * If clear option is true, this method is called. It deletes destination path.
     * User may override this method to tailor clear operation according to generator's
     * purposes.
     */
    async clear() {
        await this.deleteDestination();
    }
    /**
     * Executed after clear operation.
     */
    init() {
        return undefined;
    }
    /**
     * Provides generator specific extra context to templates. This data is deeply merged with
     * context provided by user.
     *
     * @returns generator spesific extra context data.
     *
     * @example
     * export default class App extends PgGenerator {
     *   protected context(): Record<string, any> {
     *     return { global: { addSchemaName: true } };
     *   }
     * }
     */
    context() {
        return {};
    }
    /**
     * Generates content from the template with provided path using the context.
     *
     * @param templatePath is the path of the template file to generate content.
     * @param context is the data to be used for generating content.
     * @returns generated content.
     *
     * @example
     * import engine from "my-favorite-template-engine";
     *
     * export default class Md extends PgGenerator {
     *   protected async render(templatePath: string, context: Context): Promise<string> {
     *     return engine.render(templatePath, context);
     *   }
     * }
     */
    render(templatePath, context) {
        /* istanbul ignore next */
        const stub = templatePath !== null && templatePath !== void 0 ? templatePath : context; // eslint-disable-line @typescript-eslint/no-unused-vars
        return undefined;
    }
    /**
     * Formats given content. By default `prettier` is used. Plase note that at this stage file is not written to disk yet.
     *
     * @param destination is the path of the destination file to be created.
     * @param content is the content to format.
     * @returns formatted code.
     */
    format(destination, content) {
        if (content === undefined || content === null || content === "")
            return content;
        return prettier_1.default.format(content, { filepath: destination });
    }
    /**
     * Writes generated content from [[render]] to the destination file.
     *
     * @param destination is the destionation path of the file to be cerated..
     * @param content is the content to process.
     */
    process(destination, content) {
        return this.writeTemplate(destination, content);
    }
    //
    // ────────────────────────────────────────────────────── I ──────────
    //   :::::: U T I L I T Y : :  :   :    :     :        :          :
    // ────────────────────────────────────────────────────────────────
    //
    // Protected methods below are utility methods to be used by child classes.
    /**
     * Logs given messsage if `options.log` is true. Also outputs colorized paths for
     * source and destination.
     *
     * @param title is the title of the log.
     * @param message is the nessage to log.
     * @param options are options.
     * @param options.message is the message to log.
     * @param options.source is the source path to log.
     * @param options.destination is the destination path to log.
     *
     * @hidden
     */
    /* istanbul ignore next */
    log(title, { message, source, destination }) {
        var _a;
        if (!this.options.log)
            return;
        const titleColor = (_a = COLORS[title]) !== null && _a !== void 0 ? _a : COLORS.default;
        const colorizeTitle = meow_helper_1.chalk[titleColor];
        const logTitle = colorizeTitle(title);
        const messageSpace = message && source && destination ? ": " : " ";
        const logMessage = message === undefined ? "" : ` ${message}${messageSpace}`;
        const logSource = source ? `${source}${destination ? " ➔ " : ""}` : "";
        const logDestination = destination ? path_1.join(this.options.outDir || "", destination) : "";
        console.log(`   ${logTitle} ${logMessage}${logSource}${logDestination}`); // eslint-disable-line no-console
    }
    /**
     * Returns abosulte path for the given path relative to output directory.
     *
     * @param path is the path relative to output directory.
     * @returns the absolute path.
     */
    destinationPath(path = "") {
        return path_1.resolve(__classPrivateFieldGet(this, _destinationRoot), path);
    }
    /**
     * Returns abosulte path for the given path relative to template root.
     *
     * @param path is the path relative to template root.
     * @returns the absolute path.
     */
    templatePath(path = "") {
        return path_1.resolve(__classPrivateFieldGet(this, _internalOptions).templateDir, path);
    }
    async composeWith(generator, subOrOptions, maybeOptions) {
        const [options = {}, generatorName] = typeof subOrOptions === "string" ? [maybeOptions, subOrOptions] : [maybeOptions];
        const newOptions = { ...this.options, clear: false, ...options };
        return utils_1.composeWith(generator, generatorName, newOptions, __classPrivateFieldGet(this, _internalOptions).db, __classPrivateFieldGet(this, _internalOptions).cwd);
    }
    /**
     * Writes given content to the file after creating it's directory.
     *
     * @param destinantion is the path to write file.
     * @param content is the content to write.
     */
    async writeTemplate(destination, content) {
        const destinationPath = this.destinationPath(destination);
        await fs_1.promises.mkdir(path_1.dirname(destinationPath), { recursive: true });
        await fs_1.promises.writeFile(destinationPath, content, { encoding: "utf8" });
        this.log("create", { destination });
    }
    /**
     * Copy the source file to the destination. Also creates the destination directory if it does not exist.
     *
     * @param source is the source file to copy.
     * @param destination is the destination file to be created.
     */
    async copyTemplate(source, destination) {
        await utils_1.copyFile(this.templatePath(source), this.destinationPath(destination));
        this.log("create", { destination });
    }
    /**
     * Removes file or directory (and it's contents() from destination. If destination is equal to `cwd()`
     * it throws error and aborts operation for a safety measure. Most probably `cwd()` contains other files too.
     *
     * @param path is the path relative to the destination root.
     */
    async deleteDestination(path = "") {
        const destination = this.destinationPath(path);
        const isRoot = path_1.resolve(__classPrivateFieldGet(this, _internalOptions).cwd) === path_1.resolve(destination);
        if (isRoot)
            throw new pgen_error_1.PgenError("CWD cannot be deleted, because it may be a mistake. Probably it would contain other files.");
        try {
            await fs_1.promises.rmdir(this.destinationPath(path), { recursive: true });
        }
        catch (error) {
            /* istanbul ignore next */
            if (error.code !== "ENOENT")
                throw error;
            else if (error.code === "ENOTDIR")
                await fs_1.promises.unlink(this.destinationPath(path));
        }
        this.log("delete", { destination: path || "." });
    }
    /**
     * Fetches and merges context data from `pg-generator` instance, context file and options.
     * The context file may be a JSON5 data file (may contain comments etc.) or a JavaScript file
     * with a default export of object or a function. If it is a function, it is passed
     * `pg-structure` db instance.
     *
     * Merge priorities are: `options.context` > context file > `context()` method.
     *
     * @param contextFile is the file to read from.
     * @param context is the context data.
     * @returns merged context data.
     */
    async fetchContext() {
        let contextFromFile = {};
        if (this.options.contextFile !== undefined) {
            const contextFile = path_1.resolve(this.options.contextFile);
            const fileContent = path_1.extname(contextFile) === ".json"
                ? json5_1.default.parse(await fs_1.promises.readFile(contextFile, { encoding: "utf8" }))
                : (await Promise.resolve().then(() => __importStar(require(contextFile)))).default;
            contextFromFile = typeof fileContent === "function" ? fileContent(__classPrivateFieldGet(this, _internalOptions).db) : fileContent;
            this.log("context", { destination: contextFile });
        }
        // Don't mutate original data within merge. Otherwise if same data file (as module) is used in same application, it will receive mutated data.
        return lodash_merge_1.default({}, await this.context(), contextFromFile, this.options.context);
    }
}
exports.PgGenerator = PgGenerator;
_destinationRoot = new WeakMap(), _internalOptions = new WeakMap();
//# sourceMappingURL=pg-generator.js.map