"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.scaffold = exports.readScaffolds = exports.getScaffoldsMessage = void 0;
const path_1 = require("path");
const os_1 = require("os");
const fs_1 = require("fs");
const ignor_1 = require("ignor");
const pgen_error_1 = require("./utils/pgen-error");
const utils_1 = require("./utils");
const SCAFFOLD_DIR = path_1.join(__dirname, "../module-files/scaffolds");
/**
 * Renames "name" entry of the "package.json" in given directory.
 *
 * @param dir is the directory that contains the "package.json" file.
 * @param newName is the new name.
 */
async function renameInPackageJson(dir, newName) {
    const pkgPath = path_1.join(dir, "package.json");
    const pkg = JSON.parse(await fs_1.promises.readFile(pkgPath, { encoding: "utf-8" }));
    pkg.name = newName;
    await fs_1.promises.writeFile(pkgPath, JSON.stringify(pkg, undefined, 2));
}
/**
 * Formats given list of scaffold names.
 *
 * @param scaffolds are the list of the scaffold names.
 * @internal
 */
function getScaffoldsMessage(scaffolds) {
    const available = scaffolds.join(`${os_1.EOL}  ● `);
    return `Available scaffolds are:${os_1.EOL}  ● ${available}`;
}
exports.getScaffoldsMessage = getScaffoldsMessage;
/**
 * Read list of scaffolds from disk.
 *
 * @internal
 */
async function readScaffolds() {
    return (await fs_1.promises.readdir(SCAFFOLD_DIR, { withFileTypes: true })).filter((e) => e.isDirectory()).map((e) => e.name);
}
exports.readScaffolds = readScaffolds;
/**
 * Creates a skeleton Node.js project for a generator (pg-generator plugin).
 *
 * @param scaffoldName is the name to create scaffold from.
 * @param options are options
 * @param options.outDir is the output directory to create the project.
 * @param options.name is the name of the created project.
 * @throws error if required scaffold is unavailable or output directory contains files.
 */
async function scaffold(scaffoldName, { outDir, name = `pg-generator-${path_1.basename(outDir, path_1.extname(outDir)).replace("pg-generator-", "")}` }) {
    const scaffolds = await readScaffolds();
    if (scaffoldName === undefined)
        throw new pgen_error_1.PgenError(`SCAFFOLD is required. ${getScaffoldsMessage(scaffolds)}`);
    if (!scaffolds.includes(scaffoldName))
        throw new pgen_error_1.PgenError(`Unknown scaffold "${scaffold}". ${getScaffoldsMessage(scaffolds)}`);
    try {
        const destinationEntries = await fs_1.promises.readdir(outDir).catch(ignor_1.ignoreCode("ENOENT"));
        if (destinationEntries && destinationEntries.length > 0)
            throw new pgen_error_1.PgenError(`Output directory "${outDir}" is not empty.`);
    }
    catch (error) {
        if (error.code === "ENOTDIR")
            throw new pgen_error_1.PgenError(`"${outDir}" is not a directory.`);
        throw error;
    }
    await utils_1.copyDir(path_1.join(SCAFFOLD_DIR, scaffoldName), outDir);
    await renameInPackageJson(outDir, name);
}
exports.scaffold = scaffold;
//# sourceMappingURL=scaffold.js.map