import type { Db } from "pg-structure";
import type { GeneratorOptions } from "../types";
import type { PgGenerator } from "../pg-generator";
declare type Class<T = unknown, Arguments extends any[] = any[]> = new (...arguments_: Arguments) => T;
/**
 * Imports given generator or sub-generator from given path.
 *
 * @param path is the path to import generator or sub-generaor.
 * @param generator is the generator to import.
 * @param subGenerator is the sub-generator to import.
 * @returns the generator class and it's path.
 */
export declare function importGenerator(path: string, generator: string, subGenerator?: string): Promise<[Class<PgGenerator> | undefined, string]>;
/**
 * Reads directories of first available sub-plugin path.
 *
 * @param id is the directory that contains sub-plugin paths.
 * @param subDirs possible sub-plugin paths.
 * @returns first available sub-plugin directory and sub-plugins (directories) in it. If dir is not found, returns undefined. If sub-dirs are not available, returns `[]`.
 *
 * @example
 * await readSubGenerators("my-plugin"); // ["app", "objection"]
 */
export declare function readGenerators(id: string): Promise<string[] | undefined>;
/**
 * Executes a generator or a sub-generator.
 *
 * @param generator is the generator to execute.
 * @param subGenerator is the name of the sub-generator to execute.
 * @param options are options including internal options.
 * @param cwd is the path of the destination. If the outpur directory is a relative path, it is calculated relative to the cwd.
 * @param db is the [pg-structure Db instance](https://www.pg-structure.com/nav.02.api/classes/db).
 *
 * @throws error if generator can not be found.
 *
 * @example
 * composeWith("generator-from-npm", "sub-generator-name", options, db, cwd);
 * composeWith(require.resolve("./local-generator"), "sub-generator-name", options, db, cwd);
 */
export declare function composeWith<O extends GeneratorOptions>(generator: string, subGenerator: string | undefined, options: O, db: Db, cwd: string): Promise<void>;
export {};
//# sourceMappingURL=compose-with.d.ts.map