"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.composeWith = exports.readGenerators = exports.importGenerator = void 0;
/* eslint-disable no-await-in-loop, no-restricted-syntax */
const path_1 = require("path");
const fs_1 = require("fs");
const ignor_1 = require("ignor");
const pgen_error_1 = require("./pgen-error");
const exists_1 = require("./exists");
const SPECIAL_DIRS = ["partials", "utils"];
const PREFIX = "pg-generator";
/**
 * If  given id does not have prefix, adds prefix to it.
 *
 * @param id is the id to add prefix.
 * @param prefix is the prefix to add.
 * @returns id with prefix.
 *
 * @example
 * getAlternativeId("example", "pg-generator"); // "pg-generator-example"
 * getAlternativeId("@user/example", "pg-generator"); // "@user/pg-generator-example"
 */
function getAlternativeId(id) {
    const { dir, name, ext } = path_1.parse(id);
    return name.startsWith(PREFIX) ? undefined : path_1.format({ dir, name: `${PREFIX}-${name}`, ext });
}
/**
 * Resolves path.
 * - If path is an absolute path, returns it as it is.
 * - If path is a relative path, resolves it relative to cwd.
 * - If path is a bare name (like e module name), returns dir path of the `require.resolve()`.
 *
 * @param id is the path to resolve.
 * @returns resolved absolute path.
 *
 * @example
 * resolveModule("my-module"); // /path/to/project/node_modules/my-module/dist
 * resolveModule("my-module", "pg-generator"); // /path/to/project/node_modules/my-module/dist or /path/to/project/node_modules/pg-generator-my-module/dist
 * resolveModule("/path/to/my-module"); // /path/to/my-module
 * resolveModule("./my-module"); // /path/to/my-module
 */
function resolvePath(id) {
    let result;
    // Test relative path. Matches "./",  "../",  "..\",  "..\"
    if (/^\.\.?[\\/]/.test(id))
        result = path_1.resolve(id);
    else if (path_1.isAbsolute(id))
        result = id;
    else {
        result = ignor_1.ignoreCode("MODULE_NOT_FOUND", () => require.resolve(id));
        const alternativeId = getAlternativeId(id);
        if (result === undefined && alternativeId !== undefined)
            result = ignor_1.ignoreCode("MODULE_NOT_FOUND", () => require.resolve(alternativeId));
        /* istanbul ignore next */
        return result === undefined ? undefined : path_1.dirname(result);
    }
    return result;
}
/**
 * Imports generator from given path. If imported module is not a sub-class of PgGenerator returns undefined.
 *
 * @param path is the path to import generator.
 * @param name is the name of the generator. To import named export instead of default.
 * @returns generator class.
 */
async function importGeneratorFromPath(path, name) {
    var _a, _b;
    const GeneratorModule = await Promise.resolve().then(() => __importStar(require(path))).catch(ignor_1.ignoreMessage(/Cannot find module/));
    if (GeneratorModule === undefined)
        return undefined;
    const Generator = (_a = GeneratorModule.default) !== null && _a !== void 0 ? _a : GeneratorModule[name];
    // instanceof PgGenerator is not used, because this PgGenerator may be different than PgGenerator from plugin's dependencies.
    return typeof ((_b = Generator === null || Generator === void 0 ? void 0 : Generator.prototype) === null || _b === void 0 ? void 0 : _b.generate) === "function" ? Generator : undefined;
}
/**
 * Imports given generator or sub-generator from given path.
 *
 * @param path is the path to import generator or sub-generaor.
 * @param generator is the generator to import.
 * @param subGenerator is the sub-generator to import.
 * @returns the generator class and it's path.
 */
async function importGenerator(path, generator, subGenerator) {
    var _a;
    const name = subGenerator !== null && subGenerator !== void 0 ? subGenerator : generator;
    let fullPath = path_1.join(path, subGenerator || "");
    let Generator = await importGeneratorFromPath(fullPath, name);
    if (!Generator && !subGenerator) {
        fullPath = path_1.join(path, "app");
        Generator = await importGeneratorFromPath(fullPath, name);
    }
    const isDirectory = (_a = (await fs_1.promises.lstat(fullPath).catch(ignor_1.ignoreCode(["ENOENT", "ENOTDIR"])))) === null || _a === void 0 ? void 0 : _a.isDirectory();
    if (!isDirectory)
        fullPath = path_1.dirname(fullPath);
    return [Generator, fullPath];
}
exports.importGenerator = importGenerator;
/**
 * Reads directories of first available sub-plugin path.
 *
 * @param id is the directory that contains sub-plugin paths.
 * @param subDirs possible sub-plugin paths.
 * @returns first available sub-plugin directory and sub-plugins (directories) in it. If dir is not found, returns undefined. If sub-dirs are not available, returns `[]`.
 *
 * @example
 * await readSubGenerators("my-plugin"); // ["app", "objection"]
 */
async function readGenerators(id) {
    const path = resolvePath(id);
    if (path === undefined || !(await exists_1.exists(path)))
        return undefined;
    const entries = await fs_1.promises.readdir(path, { withFileTypes: true }).catch(ignor_1.ignoreCode(["ENOTDIR", "ENOENT"], []));
    return entries.filter((entry) => entry.isDirectory() && !SPECIAL_DIRS.includes(entry.name)).map((entry) => entry.name);
}
exports.readGenerators = readGenerators;
/**
 * Executes a generator or a sub-generator.
 *
 * @param generator is the generator to execute.
 * @param subGenerator is the name of the sub-generator to execute.
 * @param options are options including internal options.
 * @param cwd is the path of the destination. If the outpur directory is a relative path, it is calculated relative to the cwd.
 * @param db is the [pg-structure Db instance](https://www.pg-structure.com/nav.02.api/classes/db).
 *
 * @throws error if generator can not be found.
 *
 * @example
 * composeWith("generator-from-npm", "sub-generator-name", options, db, cwd);
 * composeWith(require.resolve("./local-generator"), "sub-generator-name", options, db, cwd);
 */
async function composeWith(generator, subGenerator, options, db, cwd) {
    const path = resolvePath(generator);
    if (path === undefined || !(await exists_1.exists(path)))
        throw pgen_error_1.PgenError.composerError("NOGEN", generator, subGenerator);
    const [Generator, fullPath] = await importGenerator(path, generator, subGenerator);
    if (Generator === undefined)
        throw pgen_error_1.PgenError.composerError("NOSUB", generator, subGenerator, await readGenerators(generator));
    const internalOptions = { templateDir: path_1.join(fullPath, "templates"), cwd, db };
    return new Generator(options, internalOptions).generate();
}
exports.composeWith = composeWith;
//# sourceMappingURL=compose-with.js.map