import { Column } from "pg-structure";
import type { DbObject } from "pg-structure";
/**
 * Clears the default value of a database object.
 * - Converts two quotes `''` to single quote `'`
 * - Removes quotes at the start and end of string.
 * - Escapes result according to JSON standards.
 *
 * @param input is the default for a database object.
 * @returns default value to be used in a template.
 *
 * @example
 * clearDefaultValue("'No ''value'' given'"); // "No value 'given'"
 */
export declare function clearDefault(input?: string | null): string | undefined;
/**
 * Converts the given input to the camel case.
 *
 * @param input is the input string to convert.
 * @returns string as camel case.
 *
 * @example
 * camelCase("user-name"); // userName
 */
export declare function camelCase(input?: string): string;
/**
 * Converts the given input to the pascal case.
 *
 * @param input is the input string to convert.
 * @returns string as pascal case.
 *
 * @example
 * pascalCase("user-name"); // UserName
 */
export declare function pascalCase(input?: string): string;
/**
 * Converts the given input to the class name.
 *
 * @param input is the input string to convert.
 * @returns string as class case.
 *
 * @example
 * classCase("user-name"); // UserName
 */
export declare function classCase(input?: string): string;
/**
 * Converts the given input to the snake case.
 *
 * @param input is the input string to convert.
 * @returns string as snake case.
 *
 * @example
 * snakeCase("userName"); // user_name
 */
export declare function snakeCase(input?: string): string;
/**
 * Converts the given input to the dash case.
 *
 * @param input is the input string to convert.
 * @returns string as dash case.
 *
 * @example
 * dashCase("User Name"); // user-name
 */
export declare function dashCase(input?: string): string;
/**
 * Converts the given input to the title case.
 *
 * @param input is the input string to convert.
 * @returns string as title case.
 *
 * @example
 * titleCase("user_name"); // User Name
 */
export declare function titleCase(input?: string): string;
/**
 * Converts the given input to the singular form.
 *
 * @param input is the input string to convert.
 * @returns string in singular form.
 *
 * @example
 * singular("user_names"); // user_name
 */
export declare function singular(input?: string): string;
/**
 * Converts the given input to the plural form.
 *
 * @param input is the input string to convert.
 * @returns string in plural form.
 *
 * @example
 * plural("user_name"); // user_names
 */
export declare function plural(input?: string): string;
/**
 * Converts text to natural language.
 *
 * @param input is the input string to convert.
 * @param lowFirstLetter is whther to use small letter in first word.
 * @returns string in human readable form.
 *
 * @example
 * human("message_properties"); // "Message properties"
 * human("message_properties", true); // "message properties"
 */
export declare function human(input?: string, lowFirstLetter?: boolean): string;
/**
 * Converts the given input's first letter to the lower case.
 *
 * @param input is the input string to convert.
 * @returns string with lower first case.
 *
 * @example
 * plural("User_name"); // User_name
 */
export declare function lcFirst(input?: string): string;
/**
 * Converts the given input's first letter to the upper case.
 *
 * @param input is the input string to convert.
 * @returns string with upper first case.
 *
 * @example
 * plural("user_name"); // User_name
 */
export declare function ucFirst(input?: string): string;
/**
 * Wraps the given string with quotes.
 *
 * @param input is the input string to wrap with quotes.
 * @returns string with quotes.
 *
 * @example
 * plural("user_name"); // "user_name"
 */
export declare function quote(input?: string): string;
/**
 * Wraps the given string with single quotes.
 *
 * @param input is the input string to wrap with single quotes.
 * @returns string with quotes.
 *
 * @example
 * plural("Some 'example' text"); // 'some \'example\' text'
 */
export declare function singleQuote(input?: string): string;
/**
 * Wraps the given string with double quotes.
 *
 * @param input is the input string to wrap with double quotes.
 * @returns string with quotes.
 *
 * @example
 * plural("Some "example" text"); // "some \"example\" text"
 */
export declare function doubleQuote(input?: string): string;
/**
 * Vairadic function which strips all of the given strings or database object's names from the start of the source string.
 *
 * @param input is the input string to convert.
 * @param removeList is the list of strings or objects to remove from input.
 * @returns converted string.
 */
export declare function stripPrefix(input: string | undefined, ...removeList: Array<string | {
    name: string;
}>): string;
/**
 * Vairadic function which strips all of the given strings or database object's names from the end of the source string.
 *
 * @param input is the input string to convert.
 * @param removeList is the list of strings or objects to remove from input.
 * @returns converted string.
 */
export declare function stripSuffix(input: string | undefined, ...removeList: Array<string | {
    name: string;
}>): string;
/**
 * Vairadic function which strips all of the given strings or database object's names from the source string.
 *
 * @param input is the input string to convert.
 * @param removeList is the list of strings or objects to remove from input.
 * @returns converted string.
 */
export declare function strip(input: string | undefined, ...removeList: Array<string | {
    name: string;
}>): string;
/**
 * Pads given string's end with given padding string to complete its length to count.
 *
 * @param input is the input string to convert.
 * @param totalLength is the total length of the result string.
 * @param paddingString is the string to pad with.
 * @returns the string padded with padding string.
 */
export declare function padRight(input?: string, totalLength?: number, paddingString?: string): string;
/**
 * Cuts the text after given number of characters.
 *
 * @param input is the text to shorten.
 * @param length is the maximum length allowed.
 * @returns cut text
 *
 * @example
 * maxLength("some example text", 7); // "some ex...";
 */
export declare function maxLength(input?: string, length?: number): string;
/**
 * Completes given input's length using with given character (by default space). It may be used to align
 * strings in JSDoc etc.
 *
 * @param input is the input to complete length of.
 * @param length is the length of the finel string.
 * @param char is the character to be used for filling.
 *
 * @example
 * completeWithChar("member", "10"); // "member    "
 * completeWithChar("member", "10", "-"); // "member----"
 */
export declare function fill(input?: string, length?: number, char?: string): string;
/**
 * Wraps given text with start and end characters. By default it wraps with curly braces.
 *
 * @param input is the text to warp.
 * @param start is the starting string.
 * @param end is the ending string.
 * @returns wrapped text.
 *
 * @example
 * wrap("hello"); // "{hello}"
 * wrap("hello", "~"); // "~hello~"
 * wrap("hello", "[]"); // "[hello]"
 */
export declare function wrap(input?: string, wrapper?: string): string;
/**
 * Wraps given text with start and end characters if given condition is truthy.
 * By default it wraps with curly braces.
 *
 * @param input is the text to warp.
 * @param start is the starting string.
 * @param end is the ending string.
 * @param condition is the condition or value to test.
 * @returns wrapped text.
 *
 * @example
 * wrapIf("hello", "x"); // "{hello}"
 * wrapIf("hello", true); // "{hello}"
 * wrapIf("hello", false); // "hello"
 * wrapIf("hello", true, "~"); // "~hello~"
 * wrapIf("hello", true, "[]"); // "[hello]"
 */
export declare function wrapIf(input: string | undefined, condition: any, wrapper?: string): string;
/**
 * Adds given prefix each of the lines of given text.
 *
 * @param input is the input string.
 * @param prefix is the prefix to add each of the lines.
 * @returs the result string.
 *
 * @example
 * linePrefix(`
 * Text line 1
 * Text line 2
 * `, "// ");
 *
 * // Text line 1
 * // Text line 2
 */
export declare function linePrefix(input: string | undefined, prefix: string): string;
/**
 * If given data is object or array, converts it to string.
 * 1. If it has `toString` method uses it. If it returns [object Object] tries other steps.
 * 2. Uses `util.inspect()`;
 *
 * @param input is the input to convert.
 * @param options are the options.
 * @param options.raw if true, does not add quotes around values.
 * @param options.nullToUndef if true, converts all null values to undefined.
 * @param options.indent is size of the indentation of each level.
 * @returns converted value.
 */
export declare function stringify(input: any, options?: {
    nullToUndef?: boolean;
    raw?: boolean;
    indent?: number;
}): string;
/**
 * If given data is a multi line string replaces new lines with escape characters. May be used to prevent JS multi line errors.
 *
 * @param input is the input to convert.
 * @returns the string with escape characters.
 */
export declare function singleLine(input?: string): string;
/**
 * Returns given array with unique elements by eliminating duplicate values.
 *
 * @param input is the input array to eliminate duplicates from.
 * @returns the array with unique values.
 */
export declare function uniqueArray<T extends any>(input?: T[]): T[];
/**
 * Returns an attribute of each objects as a CSV (comma separated values)
 *
 * @param objects are the array of objects to get attribute of.
 * @param attribute is the attribute to get for each object.
 * @param options are the options.
 * @param options.quote is whether to add quotes around attributes.
 * @param options.join is the character to join list.
 * @param options.wrap is the characters to wrap the list if length is greater than 1.
 * @returns the list as a string.
 *
 * @example
 * const objects = [{ name: "a" }, { name: "b" }, { name: "c" }]
 *
 * listAttribute(objects, "name"); // a, b, c
 * listAttribute(objects, "name", { quote: "json" }); // "a", "b", "c"
 * listAttribute(objects, "name", { quote: "single" }); // 'a', 'b', 'c'
 * listAttribute(objects, "name", { quote: "json", wrap: "[]" }); // ["a", "b", "c"]
 * listAttribute(objects, "name", { quote: "json", wrap: "[]" }); // "a"
 */
export declare function listAttribute<T extends any>(objects: T[], attribute?: keyof T, options?: {
    quote?: "single" | "double" | "json";
    join?: string;
    wrap?: string;
}): string;
/**
 * Word wraps given text.
 *
 * @param input is the text to word wrap.
 * @param startOrStop is the start or the stop position of each line. (The stop position if this is single option.)
 * @param stop is the stop position of each line.
 * @return word wrapped text.
 *
 * @example
 * wordWrap("The quick fox", 10); // "The quick\nfox"
 * wordWrap("The quick fox", 2, 10); // "  The quick\n  fox"
 */
export declare function wordWrap(input?: string, startOrStop?: number, stop?: number): string;
/**
 * Returns given the given database object name as a class name.
 *
 * @param object is the object to get the name as a class name.
 * @param schema is whether to include schema name.
 */
export declare function dboClassName(object?: DbObject, schema?: boolean): string;
/**
 * Returns column length, precision and scale ready to be used in templates.
 *
 * @param column is the column to get details.
 * @returns modifier string.
 *
 * @example
 * columnTypeModifier(price); // (10,4)
 * columnTypeModifier(name); // (20)
 */
export declare function dboColumnTypeModifier(column?: Column): string;
//# sourceMappingURL=filter-functions.d.ts.map