"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dboColumnTypeModifier = exports.dboClassName = exports.wordWrap = exports.listAttribute = exports.uniqueArray = exports.singleLine = exports.stringify = exports.linePrefix = exports.wrapIf = exports.wrap = exports.fill = exports.maxLength = exports.padRight = exports.strip = exports.stripSuffix = exports.stripPrefix = exports.doubleQuote = exports.singleQuote = exports.quote = exports.ucFirst = exports.lcFirst = exports.human = exports.plural = exports.singular = exports.titleCase = exports.dashCase = exports.snakeCase = exports.classCase = exports.pascalCase = exports.camelCase = exports.clearDefault = void 0;
/* eslint-disable @typescript-eslint/explicit-module-boundary-types */
const os_1 = require("os");
const util_1 = require("util");
const wordwrap_1 = __importDefault(require("wordwrap"));
const inflection_1 = __importDefault(require("inflection"));
//
// ──────────────────────────────────────────────────────────────────── I ──────────
//   :::::: S T R I N G   F I L T E R S : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────────────────────────
//
/**
 * Clears the default value of a database object.
 * - Converts two quotes `''` to single quote `'`
 * - Removes quotes at the start and end of string.
 * - Escapes result according to JSON standards.
 *
 * @param input is the default for a database object.
 * @returns default value to be used in a template.
 *
 * @example
 * clearDefaultValue("'No ''value'' given'"); // "No value 'given'"
 */
function clearDefault(input) {
    // Does not support SQL functions. IMHO it is better to handle sql function default values in RDMS.
    let result = input;
    if (input === null || input === undefined)
        return undefined;
    if (input === "")
        result = "";
    else if (input.charAt(0) === "'" || input.charAt(0) === '"')
        result = input.substring(1, input.length - 1).replace(/''/g, "'");
    else if (!Number.isNaN(+input))
        return input;
    else if (input.toLowerCase() === "true")
        result = true;
    else if (input.toLowerCase() === "false")
        result = false;
    return JSON.stringify(result);
}
exports.clearDefault = clearDefault;
/**
 * Converts the given input to the camel case.
 *
 * @param input is the input string to convert.
 * @returns string as camel case.
 *
 * @example
 * camelCase("user-name"); // userName
 */
function camelCase(input = "") {
    return inflection_1.default.camelize(input.replace("-", "_"), true);
}
exports.camelCase = camelCase;
/**
 * Converts the given input to the pascal case.
 *
 * @param input is the input string to convert.
 * @returns string as pascal case.
 *
 * @example
 * pascalCase("user-name"); // UserName
 */
function pascalCase(input = "") {
    return inflection_1.default.camelize(input.replace("-", "_"), false);
}
exports.pascalCase = pascalCase;
/**
 * Converts the given input to the class name.
 *
 * @param input is the input string to convert.
 * @returns string as class case.
 *
 * @example
 * classCase("user-name"); // UserName
 */
function classCase(input = "") {
    return inflection_1.default.classify(input.replace("-", "_"));
}
exports.classCase = classCase;
/**
 * Converts the given input to the snake case.
 *
 * @param input is the input string to convert.
 * @returns string as snake case.
 *
 * @example
 * snakeCase("userName"); // user_name
 */
function snakeCase(input = "") {
    return inflection_1.default.underscore(input.replace("-", "_"));
}
exports.snakeCase = snakeCase;
/**
 * Converts the given input to the dash case.
 *
 * @param input is the input string to convert.
 * @returns string as dash case.
 *
 * @example
 * dashCase("User Name"); // user-name
 */
function dashCase(input = "") {
    return inflection_1.default.transform(input, ["underscore", "dasherize"]);
}
exports.dashCase = dashCase;
/**
 * Converts the given input to the title case.
 *
 * @param input is the input string to convert.
 * @returns string as title case.
 *
 * @example
 * titleCase("user_name"); // User Name
 */
function titleCase(input = "") {
    return inflection_1.default.titleize(input);
}
exports.titleCase = titleCase;
/**
 * Converts the given input to the singular form.
 *
 * @param input is the input string to convert.
 * @returns string in singular form.
 *
 * @example
 * singular("user_names"); // user_name
 */
function singular(input = "") {
    return input === "" ? "" : inflection_1.default.singularize(input);
}
exports.singular = singular;
/**
 * Converts the given input to the plural form.
 *
 * @param input is the input string to convert.
 * @returns string in plural form.
 *
 * @example
 * plural("user_name"); // user_names
 */
function plural(input = "") {
    return input === "" ? "" : inflection_1.default.pluralize(input);
}
exports.plural = plural;
/**
 * Converts text to natural language.
 *
 * @param input is the input string to convert.
 * @param lowFirstLetter is whther to use small letter in first word.
 * @returns string in human readable form.
 *
 * @example
 * human("message_properties"); // "Message properties"
 * human("message_properties", true); // "message properties"
 */
function human(input = "", lowFirstLetter) {
    return inflection_1.default.humanize(snakeCase(input), lowFirstLetter);
}
exports.human = human;
/**
 * Converts the given input's first letter to the lower case.
 *
 * @param input is the input string to convert.
 * @returns string with lower first case.
 *
 * @example
 * plural("User_name"); // User_name
 */
function lcFirst(input = "") {
    return input === "" ? "" : input[0].toLowerCase() + input.slice(1);
}
exports.lcFirst = lcFirst;
/**
 * Converts the given input's first letter to the upper case.
 *
 * @param input is the input string to convert.
 * @returns string with upper first case.
 *
 * @example
 * plural("user_name"); // User_name
 */
function ucFirst(input = "") {
    return input === "" ? "" : input[0].toUpperCase() + input.slice(1);
}
exports.ucFirst = ucFirst;
/**
 * Wraps the given string with quotes.
 *
 * @param input is the input string to wrap with quotes.
 * @returns string with quotes.
 *
 * @example
 * plural("user_name"); // "user_name"
 */
function quote(input) {
    return input !== undefined ? JSON.stringify(input) : "";
}
exports.quote = quote;
/**
 * Wraps the given string with single quotes.
 *
 * @param input is the input string to wrap with single quotes.
 * @returns string with quotes.
 *
 * @example
 * plural("Some 'example' text"); // 'some \'example\' text'
 */
function singleQuote(input) {
    return input !== undefined ? `'${input.replace(/'/g, "\\'")}'` : "";
}
exports.singleQuote = singleQuote;
/**
 * Wraps the given string with double quotes.
 *
 * @param input is the input string to wrap with double quotes.
 * @returns string with quotes.
 *
 * @example
 * plural("Some "example" text"); // "some \"example\" text"
 */
function doubleQuote(input) {
    return input !== undefined ? `"${input.replace(/"/g, '\\"')}"` : "";
}
exports.doubleQuote = doubleQuote;
/**
 * Vairadic function which strips all of the given strings or database object's names from the start of the source string.
 *
 * @param input is the input string to convert.
 * @param removeList is the list of strings or objects to remove from input.
 * @returns converted string.
 */
function stripPrefix(input, ...removeList) {
    if (input === undefined)
        return "";
    return removeList
        .map((item) => (typeof item === "object" ? item.name : item))
        .reduce((result, item) => result.replace(new RegExp(`^${item}[_\\s-]*`), ""), input);
}
exports.stripPrefix = stripPrefix;
/**
 * Vairadic function which strips all of the given strings or database object's names from the end of the source string.
 *
 * @param input is the input string to convert.
 * @param removeList is the list of strings or objects to remove from input.
 * @returns converted string.
 */
function stripSuffix(input, ...removeList) {
    if (input === undefined)
        return "";
    return removeList
        .map((item) => (typeof item === "object" ? item.name : item))
        .reduce((result, item) => result.replace(new RegExp(`[_\\s-]*${item}$`), ""), input);
}
exports.stripSuffix = stripSuffix;
/**
 * Vairadic function which strips all of the given strings or database object's names from the source string.
 *
 * @param input is the input string to convert.
 * @param removeList is the list of strings or objects to remove from input.
 * @returns converted string.
 */
function strip(input, ...removeList) {
    if (input === undefined)
        return "";
    return removeList
        .map((item) => (typeof item === "object" ? item.name : item))
        .reduce((result, item) => result.replace(new RegExp(`${item}[_\\s-]*`), ""), input);
}
exports.strip = strip;
/**
 * Pads given string's end with given padding string to complete its length to count.
 *
 * @param input is the input string to convert.
 * @param totalLength is the total length of the result string.
 * @param paddingString is the string to pad with.
 * @returns the string padded with padding string.
 */
function padRight(input = "", totalLength = 20, paddingString = " ") {
    if (input.length >= totalLength)
        return input;
    const count = Math.floor((totalLength - input.length) / paddingString.length);
    return input + paddingString.repeat(count);
}
exports.padRight = padRight;
/**
 * Cuts the text after given number of characters.
 *
 * @param input is the text to shorten.
 * @param length is the maximum length allowed.
 * @returns cut text
 *
 * @example
 * maxLength("some example text", 7); // "some ex...";
 */
function maxLength(input = "", length = 50) {
    return input.substring(0, length).replace(/\n/g, " ") + (input.length > length ? "..." : "");
}
exports.maxLength = maxLength;
/**
 * Completes given input's length using with given character (by default space). It may be used to align
 * strings in JSDoc etc.
 *
 * @param input is the input to complete length of.
 * @param length is the length of the finel string.
 * @param char is the character to be used for filling.
 *
 * @example
 * completeWithChar("member", "10"); // "member    "
 * completeWithChar("member", "10", "-"); // "member----"
 */
function fill(input = "", length = 20, char = " ") {
    const fillString = char.repeat(Math.max(0, length - input.length));
    return `${input}${fillString}`;
}
exports.fill = fill;
/**
 * Wraps given text with start and end characters. By default it wraps with curly braces.
 *
 * @param input is the text to warp.
 * @param start is the starting string.
 * @param end is the ending string.
 * @returns wrapped text.
 *
 * @example
 * wrap("hello"); // "{hello}"
 * wrap("hello", "~"); // "~hello~"
 * wrap("hello", "[]"); // "[hello]"
 */
function wrap(input, wrapper = "{}") {
    if (input === undefined)
        return "";
    const start = wrapper.length === 1 ? wrapper : wrapper.substring(0, wrapper.length / 2);
    const end = wrapper.length === 1 ? wrapper : wrapper.slice(-(wrapper.length / 2));
    return `${start}${input}${end}`;
}
exports.wrap = wrap;
/**
 * Wraps given text with start and end characters if given condition is truthy.
 * By default it wraps with curly braces.
 *
 * @param input is the text to warp.
 * @param start is the starting string.
 * @param end is the ending string.
 * @param condition is the condition or value to test.
 * @returns wrapped text.
 *
 * @example
 * wrapIf("hello", "x"); // "{hello}"
 * wrapIf("hello", true); // "{hello}"
 * wrapIf("hello", false); // "hello"
 * wrapIf("hello", true, "~"); // "~hello~"
 * wrapIf("hello", true, "[]"); // "[hello]"
 */
function wrapIf(input, condition, wrapper = "{}") {
    if (input === undefined)
        return "";
    return condition ? wrap(input, wrapper) : input;
}
exports.wrapIf = wrapIf;
/**
 * Adds given prefix each of the lines of given text.
 *
 * @param input is the input string.
 * @param prefix is the prefix to add each of the lines.
 * @returs the result string.
 *
 * @example
 * linePrefix(`
 * Text line 1
 * Text line 2
 * `, "// ");
 *
 * // Text line 1
 * // Text line 2
 */
function linePrefix(input = "", prefix) {
    const result = input.replace(new RegExp(`(\n|${os_1.EOL})`, "g"), `$1${prefix}`);
    return `${prefix}${result}`;
}
exports.linePrefix = linePrefix;
/**
 * Converts `null` values to `undefined` recursively.
 *
 * @private
 * @param input is the input to convert.
 * @returns null values converted to `undefined`.
 */
function _convertNullToUndef(input) {
    if (Array.isArray(input))
        return input.map((val) => _convertNullToUndef(val));
    if (input === null)
        return undefined;
    if (typeof input === "object") {
        const result = {};
        Object.keys(input).forEach((key) => (result[key] = _convertNullToUndef(input[key]))); // eslint-disable-line no-return-assign
        return result;
    }
    return input;
}
/**
 * If given data is object or array, converts it to string.
 * 1. If it has `toString` method uses it. If it returns [object Object] tries other steps.
 * 2. Uses `util.inspect()`;
 *
 * @param input is the input to convert.
 * @param options are the options.
 * @param options.raw if true, does not add quotes around values.
 * @param options.nullToUndef if true, converts all null values to undefined.
 * @param options.indent is size of the indentation of each level.
 * @returns converted value.
 */
function stringify(input, options = {}) {
    var _a;
    const result = options.nullToUndef ? _convertNullToUndef(input) : input;
    if (typeof result !== "object")
        return (_a = JSON.stringify(result, undefined, options.indent)) !== null && _a !== void 0 ? _a : "undefined";
    if (options.raw) {
        return Array.isArray(result)
            ? result.join(", ")
            : Object.entries(result)
                .reduce((reducedResult, [key, value]) => { var _a; return `${reducedResult}${key}: ${value},\n${" ".repeat((_a = options.indent) !== null && _a !== void 0 ? _a : 0)}`; }, "")
                .replace(/,\n\s*$/, "");
    }
    /* istanbul ignore next */
    return result === undefined || typeof result === "object" ? util_1.inspect(result, { depth: null }) : result.toString();
}
exports.stringify = stringify;
/**
 * If given data is a multi line string replaces new lines with escape characters. May be used to prevent JS multi line errors.
 *
 * @param input is the input to convert.
 * @returns the string with escape characters.
 */
function singleLine(input = "") {
    return input.replace(/\r\n/g, "\\r\\n").replace(/\n/g, "\\n");
}
exports.singleLine = singleLine;
/**
 * Returns given array with unique elements by eliminating duplicate values.
 *
 * @param input is the input array to eliminate duplicates from.
 * @returns the array with unique values.
 */
function uniqueArray(input) {
    return [...new Set(input)];
}
exports.uniqueArray = uniqueArray;
/**
 * Returns an attribute of each objects as a CSV (comma separated values)
 *
 * @param objects are the array of objects to get attribute of.
 * @param attribute is the attribute to get for each object.
 * @param options are the options.
 * @param options.quote is whether to add quotes around attributes.
 * @param options.join is the character to join list.
 * @param options.wrap is the characters to wrap the list if length is greater than 1.
 * @returns the list as a string.
 *
 * @example
 * const objects = [{ name: "a" }, { name: "b" }, { name: "c" }]
 *
 * listAttribute(objects, "name"); // a, b, c
 * listAttribute(objects, "name", { quote: "json" }); // "a", "b", "c"
 * listAttribute(objects, "name", { quote: "single" }); // 'a', 'b', 'c'
 * listAttribute(objects, "name", { quote: "json", wrap: "[]" }); // ["a", "b", "c"]
 * listAttribute(objects, "name", { quote: "json", wrap: "[]" }); // "a"
 */
function listAttribute(objects, attribute = "name", options = {}) {
    var _a;
    const quoters = { single: singleQuote, double: doubleQuote, json: JSON.stringify };
    const quoter = quoters[options.quote];
    const list = objects.map((o) => (quoter ? quoter(o[attribute]) : o[attribute])).join((_a = options.join) !== null && _a !== void 0 ? _a : ", ");
    return options.wrap ? wrapIf(list, objects.length > 1, options.wrap) : list;
}
exports.listAttribute = listAttribute;
/**
 * Word wraps given text.
 *
 * @param input is the text to word wrap.
 * @param startOrStop is the start or the stop position of each line. (The stop position if this is single option.)
 * @param stop is the stop position of each line.
 * @return word wrapped text.
 *
 * @example
 * wordWrap("The quick fox", 10); // "The quick\nfox"
 * wordWrap("The quick fox", 2, 10); // "  The quick\n  fox"
 */
function wordWrap(input, startOrStop = 80, stop) {
    if (input === undefined)
        return "";
    return wordwrap_1.default(startOrStop, stop)(input);
}
exports.wordWrap = wordWrap;
//
// ──────────────────────────────────────────────────────────────────────── I ──────────
//   :::::: D A T A B A S E   F I L T E R S : :  :   :    :     :        :          :
// ──────────────────────────────────────────────────────────────────────────────────
//
/**
 * Returns given the given database object name as a class name.
 *
 * @param object is the object to get the name as a class name.
 * @param schema is whether to include schema name.
 */
function dboClassName(object, schema = false) {
    if (object === undefined)
        return "";
    return classCase(schema ? object.fullName.replace(".", "_") : object.name);
}
exports.dboClassName = dboClassName;
/**
 * Returns column length, precision and scale ready to be used in templates.
 *
 * @param column is the column to get details.
 * @returns modifier string.
 *
 * @example
 * columnTypeModifier(price); // (10,4)
 * columnTypeModifier(name); // (20)
 */
function dboColumnTypeModifier(column) {
    if (column === undefined)
        return "";
    if (column.length !== undefined)
        return `(${column.length})`;
    const result = [];
    if (column.precision !== undefined)
        result.push(column.precision);
    if (column.scale !== undefined)
        result.push(column.scale);
    return result.length > 0 ? `(${result.join(", ")})` : "";
}
exports.dboColumnTypeModifier = dboColumnTypeModifier;
//# sourceMappingURL=filter-functions.js.map