"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolvePath = void 0;
const path_1 = require("path");
const lodash_get_1 = __importDefault(require("lodash.get"));
const pgen_error_1 = require("./pgen-error");
const filterFunctions = __importStar(require("./filter-functions"));
/** Store all filter functions with lower case keys. */
const LC_FILTER_FUNCTIONS = {};
// Add functions to object with lower case keys.
Object.entries(filterFunctions).forEach(([name, fn]) => (LC_FILTER_FUNCTIONS[name.toLowerCase()] = fn)); // eslint-disable-line
/**
 * Applies a filter to it's input. Filter name is case insensitive and case-type free.
 *
 * @param input is the input text to appl filter to.
 * @param filterName is the case insensitive filter name.
 * @param templatePath is the path of the template to be reported in error message.
 * @returns filtered text.
 *
 * @example
 * applyFilter("member_option", "camel-case", "template.js.njk");
 * applyFilter("member_option", "camelCase", "template.js.njk");
 * applyFilter("member_option", "cAmElCaSe", "template.js.njk");
 */
function applyFilter(input, filterName, templatePath) {
    const filterFunction = LC_FILTER_FUNCTIONS[filterFunctions.camelCase(filterName).toLowerCase()];
    if (!filterFunction)
        throw new pgen_error_1.PgenError(`There is no '${filterName}' filter: ${templatePath}`);
    return filterFunction(input);
}
/**
 * Applies an array of filters to it's input. Filter names are case insensitive and case-type free.
 *
 * @param input is the input text to appl filter to.
 * @param filterNames are case insensitive filter names.
 * @param templatePath is the path of the template to be reported in error message.
 * @returns filtered text.
 *
 * @example
 * applyFilters("member_option", ["camel-case", "pluralize"], "template.js.njk");
 * applyFilters("member_option", ["camelCase", "pluralize"], "template.js.njk");
 * applyFilters("member_option", ["cAmElCaSe", "pLuRaLiZe"], "template.js.njk");
 */
function applyFilters(input, filterNames, templatePath) {
    return filterNames.reduce((current, filterName) => applyFilter(current, filterName, templatePath), input);
}
/**
 * Resolves variable names and applies filters to the path using given context data.
 *
 * @param path is the path of the template.
 * @param dbObject is the database or database object. e.g. Table, Schema etc.
 * @returns the destination file path.
 *
 * @hidden
 * @example
 * resolvePath("x/y/my-{schema.name}{name#dash-case#plural}.js.njk", table); // x/y/my-public-member-options.js
 */
function resolvePath(path, dbObject) {
    const pattern = /\{\s*(.+?)\s*\}/g; // Match text between curly braces e.g. 'table/{name#dash-case}' captures 'name#dash-case'.
    return path
        .replace(path_1.extname(path), "") // remove extension: "{name#dash-case}.js.njk" -> "{name#dash-case}.js"
        .replace(pattern, (_, p) => {
        // resolve variables: "{name#dash-case}.js" -> "member-option.js"
        const [templateVar, ...filterNames] = p.split(/\s*#\s*/);
        const value = lodash_get_1.default(dbObject, templateVar);
        if (value === undefined || value === null)
            throw new pgen_error_1.PgenError(`'${templateVar}' cannot be found or is undefined in '${dbObject.constructor.name}'. Path is '${path}'.`);
        return applyFilters(value, filterNames, path);
    });
}
exports.resolvePath = resolvePath;
//# sourceMappingURL=resolve-path.js.map